package com.vertigrated.oss.jackson.extjs.treestore;

import com.fasterxml.jackson.core.JsonGenerator;
import com.fasterxml.jackson.databind.JsonSerializer;
import com.fasterxml.jackson.databind.SerializerProvider;
import com.vertigrated.oss.extjs.treestore.ContainerNode;
import com.vertigrated.oss.extjs.treestore.ObjectNode;
import com.vertigrated.oss.extjs.treestore.ValueNode;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.util.Iterator;

import static java.lang.String.format;

public class ContainerNodeSerializer extends JsonSerializer<ContainerNode>
{
    private static final Logger L = LoggerFactory.getLogger(ContainerNodeSerializer.class);

    @Override
    public void serialize(ContainerNode value, JsonGenerator jgen, SerializerProvider serializers) throws IOException
    {
        jgen.writeStartObject();
        jgen.writeStringField("id", value.id);
        jgen.writeStringField("name", value.name);
        jgen.writeStringField("type", value.type.getSimpleName());
        jgen.writeArrayFieldStart("children");
        final Iterator iterator = value.values.iterator();
        for (int i=0; iterator.hasNext(); i++)
        {
            final String index = format("%03d", i);
            final Object o = iterator.next();
            if (o.getClass().isAssignableFrom(String.class))
            {
                final String v = String.valueOf(o);
                jgen.writeObject(new ValueNode(value.id + "/" + index, index, String.class, value.cls, v));
            }
            else if (o.getClass().isAssignableFrom(Integer.class))
            {
                final String v = String.valueOf(o);
                jgen.writeObject(new ValueNode(value.id + "/" + index, index, Integer.class, value.cls, v));
            }
            else if (o.getClass().isAssignableFrom(Long.class))
            {
                final String v = String.valueOf(o);
                jgen.writeObject(new ValueNode(value.id + "/" + index, index, Long.class, value.cls, v));
            }
            else if (o.getClass().isAssignableFrom(Double.class))
            {
                final String v = String.valueOf(o);
                jgen.writeObject(new ValueNode(value.id + "/" + index, index, Double.class, value.cls, v));
            }
            else
            {
                final String className = o.getClass().getName();
                jgen.writeObject(new ObjectNode(value.id + "/" + index, className, o.getClass(), "object", o));
            }
        }
        jgen.writeEndArray();
        jgen.writeEndObject();
    }
}
