package com.vertigrated.oss.jackson.jdbc;

import com.fasterxml.jackson.core.JsonGenerator;
import com.fasterxml.jackson.databind.JsonSerializer;
import com.fasterxml.jackson.databind.SerializerProvider;

import javax.annotation.Nonnull;
import javax.sql.RowSet;
import javax.sql.RowSetMetaData;
import java.io.IOException;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;

public class RowSetSerializer extends JsonSerializer<RowSet>
{
    @Override
    public void serialize(@Nonnull final RowSet value, @Nonnull final JsonGenerator jgen, @Nonnull final SerializerProvider serializers) throws IOException
    {
        try
        {
            jgen.writeStartObject();
            this.serializeMetadata(value.getMetaData(), jgen);
            this.serializeRows(value, jgen);
            jgen.writeEndObject();
        }
        catch (SQLException e)
        {
            throw new RuntimeException(e);
        }
    }

    private void serializeMetadata(@Nonnull final ResultSetMetaData rsmd, @Nonnull final JsonGenerator jgen)
    {
        try
        {
            jgen.writeObjectFieldStart("metadata");
            jgen.writeArrayFieldStart("columns");
            for (int i = 0; i < rsmd.getColumnCount() + 1; i++)
            {
                jgen.writeStartObject();
                jgen.writeStringField("catalog", rsmd.getCatalogName(i));
                jgen.writeStringField("schema", rsmd.getSchemaName(i));
                jgen.writeStringField("table", rsmd.getTableName(i));
                jgen.writeStringField("name", rsmd.getColumnName(i));
                jgen.writeNumberField("ordinal", i);
                jgen.writeStringField("typeName", rsmd.getColumnTypeName(i));
                jgen.writeNumberField("type", rsmd.getColumnType(i));
                jgen.writeBooleanField("nullable", rsmd.isNullable(i) == RowSetMetaData.columnNullable);
                jgen.writeEndObject();
            }
            jgen.writeEndArray();
        }
        catch (SQLException | IOException e)
        {
            throw new RuntimeException(e);
        }
    }

    private void serializeRows(@Nonnull final RowSet rs, @Nonnull final JsonGenerator jgen)
    {
        try
        {
            jgen.writeArrayFieldStart("rows");
            while (rs.next())
            {
                this.serializeRow(rs, jgen);
            }
            jgen.writeEndArray();
        }
        catch (SQLException | IOException e)
        {
            throw new RuntimeException(e);
        }
    }

    private void serializeRow(@Nonnull final RowSet rs, @Nonnull final JsonGenerator jgen)
    {
        try
        {
            final ResultSetMetaData rsmd = rs.getMetaData();
            jgen.writeStartArray();
            for (int i = 0; i < rsmd.getColumnCount() + 1; i++)
            {
                jgen.writeString(rs.getString(i));
            }
            jgen.writeEndArray();
        }
        catch (IOException | SQLException e)
        {
            throw new RuntimeException(e);
        }
    }
}
